{-# LANGUAGE DerivingStrategies #-}
{-# LANGUAGE ScopedTypeVariables #-}
{-# LANGUAGE TypeApplications #-}
{-# LANGUAGE TypeOperators #-}
{-|
Module      : Gargantext.Core.LinearAlgebra
Description : Linear Algebra utility functions
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

Linear algebra utility functions to be used across all the Gargantext modules requiring it.
-}

module Gargantext.Core.LinearAlgebra (
  -- * Types
    Index(..)

  -- * Functions
  , createIndices

  -- * Handy re-exports
  , module Gargantext.Core.LinearAlgebra.Operations
  , module Gargantext.Core.LinearAlgebra.Distributional
  ) where

import Data.Bimap (Bimap)
import Data.Bimap qualified as Bimap
import Data.Map.Strict (Map)
import Data.Map.Strict qualified as M
import Data.Set qualified as S
import Data.Set (Set)
import Prelude
import Gargantext.Core.LinearAlgebra.Operations
import Gargantext.Core.LinearAlgebra.Distributional

newtype Index = Index { _Index :: Int }
  deriving newtype (Eq, Show, Ord, Num, Enum)

createIndices :: Ord t => Map (t, t) b -> Bimap Index t
createIndices = set2indices . map2set
  where
    map2set :: Ord t => Map (t, t) a -> Set t
    map2set cs' = foldr (\(t1, t2) s -> S.insert t1 $! S.insert t2 $! s) mempty $ M.keys cs'

    set2indices :: Ord t => Set t -> Bimap Index t
    set2indices s = foldr (uncurry Bimap.insert) Bimap.empty (zip [0..] $ S.toList s)
